#ifndef __BuddhaBoundingFrustum_H__
#define __BuddhaBoundingFrustum_H__

#include "BuddhaMatrix.h"
#include "BuddhaPlane.h"
#include "BuddhaVector3.h"
#include "BuddhaBoundingSphere.h"
#include "BuddhaBoundingContainmentType.h"
#include "BuddhaBoundingBox.h"

namespace BuddhaLib
{
	/**
	 * @internal
	 */
	enum FrustumPlane
	{
		FRUSTUM_PLANE_NEAR		= 0,
		FRUSTUM_PLANE_FAR		= 1,
		FRUSTUM_PLANE_TOP		= 2,
		FRUSTUM_PLANE_RIGHT		= 3,
		FRUSTUM_PLANE_BOTTOM	= 4,
		FRUSTUM_PLANE_LEFT		= 5
	};

	/// tX^\Ă\
	struct BoundingFrustum
	{
	protected:

		Plane mPlanes[6];

	public:
		/**
		 * @internal
		 */
		BoundingFrustum(void);

		/**
		 * @brief RXgN^
		 * @param value r[~vWFNV
		 */
		BoundingFrustum(const Matrix &value);

		/// jAv[̎擾
		Plane &Near(void)
		{
			return mPlanes[FRUSTUM_PLANE_NEAR];
		}

		/// t@[v[̎擾
		Plane &Far(void)
		{
			return mPlanes[FRUSTUM_PLANE_FAR];
		}

		/// it@[֌āj㑤̃v[̎擾
		Plane &Top(void)
		{
			return mPlanes[FRUSTUM_PLANE_TOP];
		}

		/// it@[֌ājẼv[̎擾
		Plane &Right(void)
		{
			return mPlanes[FRUSTUM_PLANE_RIGHT];
		}

		/// it@[֌āj̃v[̎擾
		Plane &Bottom(void)
		{
			return mPlanes[FRUSTUM_PLANE_BOTTOM];
		}

		/// it@[֌āj̃v[̎擾
		Plane &Left(void)
		{
			return mPlanes[FRUSTUM_PLANE_LEFT];
		}

		/**
		 * @brief	EtX^ \e position Ŏw肳W܂ł邩ǂ`FbN
		 * @param	position	`FbNW
		 * @param	result		[ʏo] ̋EtX^ \e position ̊֌W߂񋓎q̒l
		 */
		void Contains(const Vector3 &position, CONTAINMENT_TYPE &result) const;

		/**
		 * @brief	EtX^ \e position Ŏw肳W܂ł邩ǂ`FbN
		 * @param	position	`FbNW
		 * @return	̋EtX^ \e position ̊֌W߂񋓎q̒l
		 */
		CONTAINMENT_TYPE Contains(const Vector3 &position) const;

		/**
		 * @brief	EtX^ \e sphere ܂ł邩ǂ`FbN
		 * @param	sphere	Ώۂ̃XtBA
		 * @param	result	[ʏo] ̋EtX^ \e sphere ̊֌W߂񋓎q̒l
		 * @attention
		 *    oɃ|C^𓖂ĂĂ̂ŃȍԂɒӂKv
		 */
		void Contains(const BoundingSphere &sphere, CONTAINMENT_TYPE &result) const;

		/**
		 * @brief	EtX^ \e sphere ܂ł邩ǂ`FbN
		 * @attention
		 *    oɃ|C^𓖂ĂĂ̂ŃȍԂɒӂKv
		 */
		CONTAINMENT_TYPE Contains(const BoundingSphere &sphere) const;

		/**
		 * @brief	EtX^ \e box ܂ł邩ǂ`FbN
		 * @param	box		Ώۂ̋E{bNX
		 * @param	result	[ʏo] ̋EtX^ \e box ̊֌W߂񋓎q̒l
		 */
		void Contains(const BoundingBox &box, CONTAINMENT_TYPE &result) const;

		/**
		 * @brief	EtX^ \e box ܂ł邩ǂ`FbN
		 * @param	box		Ώۂ̋E{bNX
		 * @return	̋EtX^ \e box ̊֌W߂񋓎q̒l
		 */
		CONTAINMENT_TYPE Contains(const BoundingBox &box) const;
	};
}

#endif
