#ifndef __BuddhaGame_H__
#define __BuddhaGame_H__

#include <d3d9.h>
#include <list>
#include <vector>
#include <dinput.h>

#include "BuddhaGameTime.h"
#include "BuddhaGameComponent.h"
#include "BuddhaGameServiceContainer.h"
#include "BuddhaGameWindow.h"

namespace BuddhaLib
{
	class GameComponentCollection;
	class GraphicsDeviceManager;
	class GameListener;

	/// {IȃOtBbNfoCX̏AQ[WbNA_OR[h񋟂
	class Game
	{
		friend GraphicsDeviceManager;

	private:
		static Game *_mInstance;

		GameTime	mStartTime;	///< Q[Jn

		GameTime		mGameTime;
		LPDIRECTINPUT8	m_pInput;

		DWORD	mGameStartTime;	///< Q[NJn
		DWORD	mLastUpdateTime;	///< Ō Update ̋L^

		bool	mHasTimeInitialized;	///< mLastUpdateTime L^ǂ

		bool	mIsFixedTimeStep;	///< ŒXVǂ

		DWORD		mTargetElapsedTime;		///< Œt[̖ڕWXVԊui~bj
		TimeSpan	mTargetElapsedTimeSpan;	///< Œt[̖ڕWXVԊu߂ TimeSpan \

		GraphicsDeviceManager	*m_pGraphicDeviceManagerInterface;

		bool	mIsActive;	///< AvP[VANeBuǂ

		friend	GraphicsDeviceManager;

		/// ֘AtꂽIy[VVXẽEBhE
		GameWindow*	m_pGameWindow;	

		typedef std::list<GameListener*> GameListenerList;
		GameListenerList mGameListenerList;

	protected:
		/// R|[lgRNVւ̃|C^
		GameComponentCollection* m_pComponents;

		/**
		 * @iternal
		 */
		GameServiceContainer* m_pServices;

		virtual HRESULT initD3D();
		virtual HRESULT initInput();

		/**
		 * @internal
		 * @brief Windows OS ̃ZbgAbvs
		 */
		virtual void initializeForWindows(void);

		/**
		 * @brief Q[̏s
		 * @attention
		 *      ̎_ł̓OtBbNfoCX͏ĂȂ߁Aӂ邱
		 */
		virtual void Initialize(void) = 0;

	public:
		/// RXgN^
		Game(void);

		/// fXgN^
		virtual ~Game();

		/**
		 * @internal
		 */
		static Game* _getInstance(void);

		/**
		 * @internal
		 */
		LPDIRECTINPUT8 _getDirectInput8(void);

		void Run(void);
		void _Update(void);
		void _Draw(void);

		/// Q[WbN̏KvȂƂɌĂяoBI[o[ChāAQ[EWbNB
		virtual void Update(const GameTime &gameTime) = 0;

		/// Pt[̕`悪KvȂƂɌĂяoBI[o[ChāA_OR[hB
		virtual void Draw(const GameTime &gameTime) = 0;

		/**
		 * @brief Q[OtBbNRec[h
		 * @remarks
		 *     ̃o֐́AKvɉĊNXĂт܂B
		 *
		 *     \li Game::Update() ̃[vJnOɁA loadAllContent = true ŁÃo֐
		 *         Ăт܂B̍ہAR|[lgRNVɃRNV΁AɃRN
		 *         V DrawableGameComponent::LoadGraphicContent() Ăт܂B
		 *     \li foCX̏ȂǂɂAxOtBbNRecŊmۂȂ΂ȂȂȂ
		 *         ꍇAloadAllContent = false ł̊֐Ăт܂B̏ꍇAɃR|[l
		 *         g̃[hsAŌ Game NX̃A[hĂт܂B
		 *
		 * @param loadAllContent true ̂ƂAǗ̃OtBbNRec[h
		 */
		virtual void LoadGraphicsContent(bool loadAllContent) {}

		/**
		 * @brief Q[ĂOtBbNRec
		 * @remarks
		 *     ̃o֐́AKvɉĊNXĂт܂B
		 *
		 *     \li foCX̏ȂǂɂAxOtBbNRecȂ΂ȂȂꍇA
		 *         unloadAllContent = false ł̊֐Ăт܂B̏ꍇAɃR|[lg
		 *         RNṼA[hsAŌ Game NX̃A[hĂт܂B
		 *     \li AvP[VIۂɁA unloadAllContent = true ł̊֐Ăт܂B
		 *         ̏ꍇAR|[lgRNṼA[h͌Ăт܂B
		 *
		 * @param unloadAllContent true ̂ƂAǗ̃OtBbNRec
		 */
		virtual void UnloadGraphicsContent(bool unloadAllContent) {}

		virtual void Cleanup(void) {}

		/// R|[lgRNV擾
		inline GameComponentCollection* GetComponents(void) const;

		/// ̃Q[݂̃ANeBuAvP[Vǂ
		inline bool IsActive(void);

		/**
		 * @internal
		 * @brief IsActive ̒lZbg
		 * @attention
		 *     pr
		 */
		void _isActive(bool isActive);

		/**
		 * @iternal
		 */
		inline GameServiceContainer* GetServices(void) const;

		/**
		 * @brief {IȃIy[VVXẽEBhE擾
		 */
		inline GameWindow* GetWindow(void) const
		{
			return m_pGameWindow;
		}

		/**
		 * @internal
		 */
		void _addGameListener(GameListener *listner);

		void _removeGameListener(GameListener *listner);

	protected:
		virtual bool BeginDraw(void);
		virtual void EndDraw(void);
	};

	/**
	 * @internal
	 */
	class GameListener
	{
	public:
		/**
		 * @internal
		 * @brief Rc֌W̏IƂɒʒm
		 */
		virtual void _notifyFinishedInit3D(void) = NULL;
	};
}

#endif