#ifndef __BuddhaMatrix_H__
#define __BuddhaMatrix_H__

#include <d3dx9.h>
#include "BuddhaQuaternion.h"

namespace BuddhaLib
{
	struct Vector3;
	struct Quaternion;

	/// S~Ss
	struct Matrix
	{
	public:
		FLOAT M11;	///< PsP̒l
		FLOAT M12;	///< PsQ̒l
		FLOAT M13;	///< PsR̒l
		FLOAT M14;	///< PsS̒l

		FLOAT M21;	///< QsP̒l
		FLOAT M22;	///< QsQ̒l
		FLOAT M23;	///< QsR̒l
		FLOAT M24;	///< QsS̒l

		FLOAT M31;	///< RsP̒l
		FLOAT M32;	///< RsQ̒l
		FLOAT M33;	///< RsR̒l
		FLOAT M34;	///< RsS̒l

		FLOAT M41;	///< SsP̒l
		FLOAT M42;	///< SsQ̒l
		FLOAT M43;	///< SsR̒l
		FLOAT M44;	///< SsS̒l

		static const Matrix Identity;

		Matrix()
		{
			M11 = M12 = M13 = M14 =
			M21 = M22 = M23 = M24 =
			M31 = M32 = M33 = M34 =
			M41 = M42 = M43 = M44 = 0.0f;
		}

		Matrix(FLOAT m11, FLOAT m12, FLOAT m13, FLOAT m14, FLOAT m21, FLOAT m22, FLOAT m23, FLOAT m24,
			FLOAT m31, FLOAT m32, FLOAT m33, FLOAT m34, FLOAT m41, FLOAT m42, FLOAT m43, FLOAT m44)
		{
			M11 = m11;
			M12 = m12;
			M13 = m13;
			M14 = m14;

			M21 = m21;
			M22 = m22;
			M23 = m23;
			M24 = m24;

			M31 = m31;
			M32 = m32;
			M33 = m33;
			M34 = m34;

			M41 = m41;
			M42 = m42;
			M43 = m43;
			M44 = m44;
		}

		inline Matrix& operator = (const D3DMATRIX &value)
		{
			M11 = value._11;
			M12 = value._12;
			M13 = value._13;
			M14 = value._14;

			M21 = value._21;
			M22 = value._22;
			M23 = value._23;
			M24 = value._24;

			M31 = value._31;
			M32 = value._32;
			M33 = value._33;
			M34 = value._34;

			M41 = value._41;
			M42 = value._42;
			M43 = value._43;
			M44 = value._44;

			return *this;
		}

		inline Matrix& operator = (const D3DXMATRIX &value)
		{
			M11 = value._11;
			M12 = value._12;
			M13 = value._13;
			M14 = value._14;

			M21 = value._21;
			M22 = value._22;
			M23 = value._23;
			M24 = value._24;

			M31 = value._31;
			M32 = value._32;
			M33 = value._33;
			M34 = value._34;

			M41 = value._41;
			M42 = value._42;
			M43 = value._43;
			M44 = value._44;

			return *this;
		}

		inline Matrix operator * (const Matrix &value)
		{
			return Multiply(*this, value);
		}

		friend inline Matrix operator * (const Matrix &left, const Matrix &right)
		{
			return Multiply(left, right);
		}

		/// Matrix^D3DMATRIX^ɕϊ
		static void CreateD3DXMatrix(const Matrix &in, D3DMATRIX &out)
		{
			out._11 = in.M11;
			out._12 = in.M12;
			out._13 = in.M13;
			out._14 = in.M14;

			out._21 = in.M21;
			out._22 = in.M22;
			out._23 = in.M23;
			out._24 = in.M24;

			out._31 = in.M31;
			out._32 = in.M32;
			out._33 = in.M33;
			out._34 = in.M34;

			out._41 = in.M41;
			out._42 = in.M42;
			out._43 = in.M43;
			out._44 = in.M44;
		}

		/**
		 * @brief D3DMATRIX^Matrix^ɕϊ
		 * @remarks
		 *     ZłsƂł邪AI BuddhaLib  DirectX ւ̈ˑ
		 *     Ȃꍇi񂩁HjlɓāÃ[eBeB֐gׂłB
		 */
		static void CreateFromD3DXMatrix(const D3DMATRIX &in, Matrix &out)
		{
			out = in;
		}

		/**
		 * @brief _Wƒ_Ww肵ăr[s쐬
		 * @todo OZs
		 */
		static Matrix CreateLookAt(const Vector3 &from, const Vector3 &to, const Vector3 &up);

		/// ˉes쐬ĕԂ
		static Matrix CreatePerspectiveFieldOfView(FLOAT fieldOfView, FLOAT aspectRatio, FLOAT nearPlaneDistance, FLOAT farPlaneDistance);

		/// ˉes쐬
		static void CreatePerspectiveFieldOfView(FLOAT fieldOfView, FLOAT aspectRatio, FLOAT nearPlaneDistance, FLOAT farPlaneDistance, Matrix &result);

		/// X̉]}gNX쐬
		static void CreateRotationX(FLOAT radians, Matrix &out);

		/// X̉]}gNX쐬ĕԂ
		static Matrix CreateRotationX(FLOAT radians);

		/// Ỷ]}gNX쐬
		static void CreateRotationY(FLOAT radians, Matrix &out);

		/// Ỷ]}gNX쐬ĕԂ
		static Matrix CreateRotationY(FLOAT radians);

		/// Z̉]}gNX쐬
		static void CreateRotationZ(FLOAT radians, Matrix &out);

		/// Z̉]}gNX쐬ĕԂ
		static Matrix CreateRotationZ(FLOAT radians);

		/// XP[O}gNX쐬
		static void CreateScale(const FLOAT sx, const FLOAT sy, const FLOAT sz, Matrix &out);

		/// XP[O}gNX쐬
		static void CreateScale(const FLOAT scale, Matrix &out);

		/// XP[O}gNX쐬ĕԂ
		static Matrix CreateScale(const FLOAT sx, const FLOAT sy, const FLOAT sz);

		/// XP[O}gNX쐬ĕԂ
		static Matrix CreateScale(const FLOAT scale);

		/// XP[O}gNX쐬ĕԂ
		static Matrix CreateScale(const Vector3 &vec);

		/// gXtH[s쐬
		static void CreateTransform(const FLOAT x, const FLOAT y, const FLOAT z, Matrix &out);

		/// gXtH[s쐬ĕԂ
		static Matrix CreateTransform(const FLOAT x, const FLOAT y, const FLOAT z);

		/// gXtH[s쐬ĕԂ
		static Matrix CreateTransform(const Vector3 &vec);

		/// s
		static void Multiply(const Matrix &mat1, const Matrix &mat2, Matrix &result);

		/// sĕԂ
		static Matrix Multiply(const Matrix &mat1, const Matrix &mat2);

		/// NH[^jI]}gNX쐬
		static void CreateFromQuaternion(const Quaternion &quaternion, Matrix &out);

		/// NH[^jI]}gNX쐬ĕԂ
		static Matrix CreateFromQuaternion(const Quaternion &quaternion);

		/**
		 * @brief w肳ꂽIuWFNgW̎J։]r{[h쐬܂B
		 * @param objectPosition IuWFNgW
		 * @param cameraPosition JW
		 * @param cameraUpVector J̏ւ̃xNg
		 * @return r{[h]}gNX
		 */
		static Matrix CreateBillboard(const Vector3 &objectPosition, const Vector3 &cameraPosition,
			const Vector3 &cameraUpVector);

		/**
		 * @brief w肳ꂽIuWFNgW̎J։]r{[h쐬܂B
		 * @param objectPosition IuWFNgW
		 * @param cameraPosition JW
		 * @param cameraUpVector J̏ւ̃xNg
		 * @param result ʂ󂯎}gNXւ̎Q
		 * @return r{[h]}gNX
		 * @todo
		 *    DXwp֐gĂ̂ŁAƂŌŏC
		 */
		static void CreateBillboard(const Vector3 &objectPosition, const Vector3 &cameraPosition,
			const Vector3 &cameraUpVector, Matrix &result);
	};
}


#endif