#ifndef __BuddhaVertexDeclaration_H__
#define __BuddhaVertexDeclaration_H__

#include <d3d9.h>
#include "BuddhaVector2.h"
#include "BuddhaVector3.h"
#include "BuddhaVector4.h"
#include "BuddhaSharedPtr.h"
#include "BUddhaColor.h"

namespace BuddhaLib
{
	class GraphicsDevice;

	class VertexDeclaration
	{
		friend GraphicsDevice;
		
	private:
		GraphicsDevice* m_pDevice;
		LPDIRECT3DVERTEXDECLARATION9 m_pVertexDeclaration;

	public:
		VertexDeclaration(GraphicsDevice* graphicDevice, const D3DVERTEXELEMENT9* elements);
	};

	/// VertexDeclaration NXp̋L|C^NX
	class VertexDeclarationPtr : public SharedPtr<VertexDeclaration>
	{
	public:
		VertexDeclarationPtr() : SharedPtr<VertexDeclaration>() {}
		explicit VertexDeclarationPtr(VertexDeclaration* rep) : SharedPtr<VertexDeclaration>(rep) {}
		VertexDeclarationPtr(const VertexDeclarationPtr &r) : SharedPtr<VertexDeclaration>(r) {}
	protected:
		void destory(void)
		{
			SharedPtr<VertexDeclaration>::destroy();
		}
	};

	struct VertexPositionColor
	{
	public:
		Vector3 Position;
		PackedColor Color;

		static const D3DVERTEXELEMENT9 VertexElements[];
		static const int SizeInBytes;

		VertexPositionColor();
	};

	struct VertexPositionNormalTexture
	{
	public:
		Vector3 Position;
		Vector3 Normal;
		Vector2 TextureCoordinate;

		static const D3DVERTEXELEMENT9 VertexElements[];
		static const int SizeInBytes;

		VertexPositionNormalTexture();
		VertexPositionNormalTexture(const Vector3& position, const Vector3& normal, const Vector2& texcoord);
	};

	struct VertexPositionNormalBlendColorTexture
	{
	public:
		Vector3 Position;
		Vector3 Normal;
		Vector4 BlendWeights;
		Vector4 BlendIndices;
		D3DCOLOR Color;
		Vector2 TextureCoordinate;

		static const D3DVERTEXELEMENT9 VertexElements[];
		static const int SizeInBytes;

		VertexPositionNormalBlendColorTexture();
		//VertexPositionNormalBlendColorTexture(const Vector3& position, const Vector3& normal, const Vector2& texcoord);
	};
}

#endif