#ifndef __BuddhaExampleCamera_H__
#define __BuddhaExampleCamera_H__

#include "../BuddhaVector3.h"
#include "../BuddhaMatrix.h"
#include "../BuddhaBoundingFrustum.h"

using namespace BuddhaLib;

/**
 * @brief wb_t@CŎgvg^CsOp̊ȒPȃR[hW
 * @remarks
 *     ̖OԂɑNX̓NXCüQƂ͈قȂB
 *     {iIɎgpꍇ́A炩߃\[XR[hRs[ĎgĂB
 *     API ƂẲiX͈ؕۏႳȂB
 */
namespace BuddhaLibExample
{
	/**
	 * @brief ɃVvȃJ
	 *
	 * JWƒ_ݒ肵A̃}gNX擾Ďgp܂B
	 * \code
	 *   BuddhaXNAExample::ExampleCamera* pCamera = new BuddhaXNAExample::ExampleCamera();
	 *
	 *   pCamera->SetPosition(0, 50.0f, 100.0f);
	 *   pCamera->SetLookAt(0, 0, 0);
	 *
	 *   // pAeBgȂǂ͉]
	 *   pCamera->pitch((FLOAT)MathHelper::ToRadians(45.0f));
	 *
	 *   Matrix viewMat = pCamera->GetViewMatrix();
	 *   Matrix projMat = pCamera->GetProjectionMatrix();
	 * \endcode
	 *
	 * @attention
	 *     BuddhaXNAExample OԂ̓ÃNX̓fXgN^ȊOŉz֐ĂȂB
	 */
	class ExampleCamera
	{
	protected:
		/// JW
		Vector3 mPosition;

		/// J̎p
		Quaternion mOrientation;

		/// jAENbvEfBX^Xil1.0fj
		FLOAT mNDistance;

		/// t@[ENbvEfBX^Xil1000.0fj
		FLOAT mFDistance;

		/// tB[hEIuEr[il STxj
		FLOAT mFOV;

		/// AXyNgEVIil 1.33j
		FLOAT mAspectRatio;

		/// r[}gNX̃LbV
		mutable Matrix mView;

		/// vWFNV}gNX̃LbV
		mutable Matrix mProj;

		/// r[}gNX̃LbVL/tO
		mutable bool mOutOfDateViewMatrix;

		/// vWFNV}gNX̃LbVL/tO
		mutable bool mOutOfDateProjMatrix;

		/// r[tX^̃LbV
		mutable BoundingFrustum mFrustum;

		/// r[tX^̃LbVL/tO
		mutable bool mOutOfDateFrustum;

	public:
		/// RXgN^
		ExampleCamera(void)
			: mPosition(Vector3::Zero),
			  mFOV((FLOAT)MathHelper::ToRadians(45.0f)),
			  mNDistance(1.0f),
			  mFDistance(1000.0f),
			  mOrientation(Quaternion::Identity),
			  mOutOfDateViewMatrix(true),
			  mOutOfDateProjMatrix(true),
			  mOutOfDateFrustum(true)
		{
			mFOV = (FLOAT)MathHelper::ToRadians(45.0f);
			mAspectRatio = 640.0f / 480.0f;
		}

		/// fXgN^
		virtual ~ExampleCamera(void)
		{
		}

		/// JW̐ݒ
		void SetPosition(const Vector3 &pos)
		{
			mPosition = pos;
			mOutOfDateViewMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// JW̐ݒ
		void SetPosition(FLOAT x, FLOAT y, FLOAT z)
		{
			mPosition.X = x;
			mPosition.Y = y;
			mPosition.Z = z;
			mOutOfDateViewMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// JW̎擾
		const Vector3 &GetPosition(void) const
		{
			return mPosition;
		}

		/// FOV̐ݒ
		void SetFOV(FLOAT fov)
		{
			mFOV = fov;
			mOutOfDateProjMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// AXyNgEVI̐ݒ
		void SetAspectRatio(FLOAT ratio)
		{
			mAspectRatio = ratio;
			mOutOfDateProjMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// jAENbvEfBX^X̐ݒ
		void SetNearDistance(FLOAT nearDistance)
		{
			mNDistance = nearDistance;
			mOutOfDateProjMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// t@[ENbvEfBX^X̐ݒ
		void SetFarDistance(FLOAT farDistance)
		{
			mFDistance = farDistance;
			mOutOfDateProjMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// _փJ
		void SetLookAt(const Vector3 &lookAt)
		{
			mOrientation = MathHelper::CreateQuaternionRotationTo(mPosition, lookAt);
			mOutOfDateViewMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// _փJ
		void SetLookAt(FLOAT x, FLOAT y, FLOAT z)
		{
			mOrientation = MathHelper::CreateQuaternionRotationTo(mPosition, Vector3(x, y, z));
			mOutOfDateViewMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// wpxiWAjɃ[]
		void Yaw(FLOAT angle)
		{
			mOrientation *= Quaternion::CreateFromAxisAngle(Vector3::UnitY, angle);
			mOutOfDateViewMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// wpxiWAjɃsb`]
		void Pitch(FLOAT angle)
		{
			mOrientation *= Quaternion::CreateFromAxisAngle(Vector3::UnitX, angle);
			mOutOfDateViewMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// wpxiWAjɃ[]
		void Roll(FLOAT angle)
		{
			mOrientation *= Quaternion::CreateFromAxisAngle(Vector3::UnitZ, angle);
			mOutOfDateViewMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// wxNg̕փJʒuړiJ̎p̉e󂯂...[JWj
		void Translate(const Vector3 &transVec)
		{
			mPosition += Vector3::Transform(transVec, mOrientation);
			mOutOfDateViewMatrix = true;
			mOutOfDateFrustum = true;
		}

		/// r[}gNX̎擾
		const Matrix &GetViewMatrix(void) const
		{
			if (mOutOfDateViewMatrix)
			{
				mView = Matrix::CreateLookAt(mPosition, mPosition + Vector3::Transform(Vector3::Forward, mOrientation), Vector3::Up);
				mOutOfDateViewMatrix = false;
			}

			return mView;
		}

		/// vWFNV}gNX̎擾
		const Matrix &GetProjectionMatrix(void) const
		{
			if (mOutOfDateProjMatrix)
			{
				mProj = Matrix::CreatePerspectiveFieldOfView(mFOV, mAspectRatio, mNDistance, mFDistance);
				mOutOfDateProjMatrix = false;
			}

			return mProj;
		}

		const BoundingFrustum &GetBoundingFrustum(void) const
		{
			if (mOutOfDateFrustum)
			{
				mFrustum = BoundingFrustum(GetViewMatrix() * GetProjectionMatrix());
				mOutOfDateFrustum = false;
			}

			return mFrustum;
		}
	};
}


#endif